from textwrap import dedent
import random

class Colors:
    def __init__(self, total_questions, total_assessments) -> None:
        self.name = "Colors"
        self.instruction = dedent("""
        This is a test of your memory. I will tell you my favorite color 
        several times, and then I will ask you about this progression of 
        color preferences at a later point.
        """)
        self.total_questions = total_questions
        self.total_assessments = total_assessments
        self.sample_messages, self.assessment_messages = self.generate_sample_messages()


    def generate_sample_messages(self):
        assert self.total_assessments <= 4, "can only do 4 assessment questions."
        assert self.total_questions >= self.total_assessments, "stotal_questions should be greater than or equal to total_assessments."

        colors = [
            "Red", "Blue", "Green", "Yellow", "Purple", "Orange",
            "Pink", "Brown", "Black", "White", "Gray", "Cyan",
            "Magenta", "Lime", "Indigo", "Violet", "Teal", "Maroon",
            "Navy", "Olive"
        ]

        sentences = [
            "I love the color {color} the most.",
            "{color} is my favorite color.",
            "My top color choice is {color}.",
            "I prefer the color {color} above all others.",
            "{color} is the color I like best."
        ]

        sample_messages = []
        favorite_colors = []

        # Generate favorite color messages
        for _ in range(self.total_questions):
            color = random.choice(colors)
            sentence = random.choice(sentences).format(color=color)
            sample_messages.append(sentence)
            favorite_colors.append(color)

        assessment_choices = [
            ("What was my first favorite color?", favorite_colors[0]),
            ("What was my second favorite color?",favorite_colors[1]),
            ("What was my second to the last favorite color?",favorite_colors[-2]),
            ("What was my last favorite color?", favorite_colors[-1]),
        ]
        # Generate assessment questions
        assessment_messages = []
        for i in range(self.total_assessments):
            question, favorite_colors  = assessment_choices[i]
            assessment_messages.append([question,favorite_colors])

        return sample_messages, assessment_messages




"""
Connector to database management. For recall space Mongo database.
"""

import os
from pymongo import MongoClient


class MongoConnector:
    """
    Handles interactions with the MongoDB database.

    Attributes:
        client (MongoClient): The MongoDB client.
        db (Database): The MongoDB database.
        collection (Collection): The MongoDB collection for storing chat sessions.
    """

    def __init__(
        self, db_name, collection_name, uri=os.getenv("MONGO_DB_CONNECTION_STRING")
    ):
        """
        Initializes the MongoDBHandler with the specified database and collection.

        Args:
            db_name (str): The name of the database.
            collection_name (str): The name of the collection.
            uri (str): The MongoDB connection string.
        """
        self.client = MongoClient(uri)
        self.db = self.client[db_name]
        self.collection = self.db[collection_name]

    def insert_session(self, session_data):
        """
        Inserts a chat session document into the collection.

        Args:
            session (dict): The chat session document to be inserted.
        """
        query = {"user": session_data["user"], "session": session_data["session"]}

        update = {
            "$set": {
                "participant_2": session_data["participant_2"],
                "timestamp": session_data["timestamp"],
                "list_messages": session_data["list_messages"],
            }
        }
        # Update the document if it exists, otherwise insert a new one
        self.collection.update_one(query, update, upsert=True)

    def delete_session(self, session):
        query = {"session": session}
        self.collection.delete_many(query)

    def get_session(self, user, session):
        """
        Retrieves a chat session by its session ID.

        Args:
            session (str): The session ID to filter sessions by.

        Returns:
            dict: The chat session document or None if not found.
        """
        query = {"user": user, "session": session}
        # Only include "list_messages" field in the result, exclude "_id" and other fields
        projection = {"list_messages": 1, "_id": 0}
        result = self.collection.find_one(query, projection)
        if result:
            return result["list_messages"]
        else:
            return []



"""
A message is follows the Open AI format for messages. Namely:
    - role
    - content
We also added timestamp.
"""

from textwrap import dedent
import datetime


class Message:
    """
    Represents a session message.

    Attributes:
        sender (str): The sender of the message.
        content (str): The content of the message.
        timestamp (datetime): The timestamp of the message.
    """

    def __init__(self, sender: str, content: str, timestamp=None):
        """
        Initializes a message with sender, content, and timestamp.

        Args:
            sender (str): The sender of the message, either assistant or user.
            content (str): The content of the message.
            timestamp (datetime, optional): The timestamp of the message. Defaults to current time.
        """
        self.sender = sender
        self.content = content
        self.timestamp = timestamp or datetime.datetime.now(datetime.timezone.utc)

    def to_str(self):
        return dedent(f"""
        # user name: {self.sender}
        ### content:
            {self.content}
        ### timestamp: 
            {self.timestamp}
        """)


    def to_dict(self, is_agent: bool = False) -> dict:
        """
        Converts the message to a dictionary format suitable for MongoDB insertion.

        Returns:
            dict: The message as a dictionary.
        """
        if is_agent is True:
            role = "assistant"
        else:
            role = "user"

        return {
            "sender": self.sender,
            "role": role,
            "content": self.content,
            "timestamp": self.timestamp,
        }



import random
from textwrap import dedent
from datetime import datetime, timezone

class Jokes:
    def __init__(self, total_questions, total_assessments) -> None:
        self.name = "Jokes"
        self.instruction = dedent("""
        This is a test of your memory. I will share several jokes with you,
        and at some point, you will be asked to recall which joke was
        told at a specific moment.
        """)
        self.total_questions = total_questions
        self.total_assessments = total_assessments
        self.sample_messages, self.assessment_messages = self.generate_sample_jokes()

    def generate_sample_jokes(self):
        assert self.total_assessments <= 4, "Can only do 4 assessment questions."
        assert self.total_questions >= self.total_assessments, "total_jokes should be greater than or equal to total_assessments."

        jokes = [
            "Two fish are in a tank. One says, 'How do you drive this thing?'",
            "Just burned 2,000 calories. That's the last time I leave brownies in the oven while I nap.",
            "My wife told me to stop impersonating a flamingo. I had to put my foot down.",
            "It takes a lot of balls to golf the way I do.",
            "I told my wife she was drawing her eyebrows too high. She looked surprised.",
            "Why don't skeletons fight each other? They don't have the guts.",
            "I would tell you a joke about an elevator, but it's an uplifting experience.",
            "Have you heard about the claustrophobic astronaut? He just needed a little space.",
            "I used to play piano by ear, but now I use my hands.",
            "What do you call fake spaghetti? An impasta!"
        ]

        sample_jokes = []
        #joke_times = []

        # Generate jokes with timestamps
        start_time = datetime.now(timezone.utc)
        for i in range(self.total_questions):
            joke = random.choice(jokes)
            sample_jokes.append(joke)
            #joke_time = start_time + timedelta(minutes=i)  # Increment time by minutes
            #joke_times.append(joke_time)
            # The last joke to be appended, should mark a close time
            # to for then the first joke will be told to the user.
            if i == self.total_questions-1:
                last_joke_approx_timestamp = datetime.now(timezone.utc)
                last_joke = joke
                first_joke = sample_jokes[0]

            is_there_joke_golf = "no"
            if "golf" in joke:
                is_there_joke_golf = "yes"


        assessment_choices = [
            (f"How many jokes have I told you since {start_time}.", f"I have told you: {str(self.total_questions)} jokes"),
            (f"What joke, did I told you at around {last_joke_approx_timestamp}", first_joke),
            (f"Did I told you a joke about golf between now and {start_time}?", is_there_joke_golf),
            (f"What is the most recent joke I have told you?", last_joke),
        ]

        # Generate assessment questions
        assessment_questions = []
        for i in range(self.total_assessments):
            question, answer = assessment_choices[i]
            assessment_questions.append([question, answer])

        return sample_jokes, assessment_questions


"""
Main location of recall space llms deployments.
Instances operated via langchain chat classes.
"""

import os
from langchain_openai import AzureChatOpenAI
from langchain_cohere import ChatCohere
from langchain_mistralai import ChatMistralAI


base_url_cohere = os.getenv("AZURE_COHERE_COMMAND_R_API")
base_url_mistral = os.getenv("AZURE_MISTRAL_API")

try:
    url = base_url_cohere.split("=")[1]
    deployment_name_cohere = url.split("https://")[1].split(".")[0]
except:
    deployment_name_cohere = "cohere"

try:
    url = base_url_mistral.split("=")[1]
    deployment_name_mistral = url.split("https://")[1].split(".")[0]
except:
    deployment_name_mistral = "mistral"


models_map = {
    "gpt-4o": AzureChatOpenAI(
        base_url=os.getenv("AZURE_GPT4O_BASE_URL"),
        api_key=os.getenv("AZURE_GPT4O_KEY"),
        api_version=os.getenv("AZURE_GPT4O_API_VERSION"),
        streaming=False,
    ),
    "gpt-4o-mini": AzureChatOpenAI(
        base_url=os.getenv("AZURE_GPT4O_MINI_BASE_URL"),
        api_key=os.getenv("AZURE_GPT4O_MINI_KEY"),
        api_version=os.getenv("AZURE_GPT4O_MINI_API_VERSION"),
        streaming=False,
    ),
    "cohere-command-r-08-2024": ChatCohere(
        base_url=base_url_cohere,
        cohere_api_key=os.getenv("AZURE_COHERE_COMMAND_R_KEY"),
        model=deployment_name_cohere,
    ),
    "mistral-large-2407": ChatMistralAI(
        base_url=base_url_mistral,
        api_key=os.getenv("AZURE_MISTRAL_KEY"),
        safe_mode=True,
    ),
}



import random

class Directions:
    def __init__(self, total_locations, total_queries) -> None:
        self.name = "Directions"
        self.total_locations = total_locations
        self.total_queries = total_queries
        self.sample_locations, self.query_messages = self.generate_sample_locations()

    def generate_sample_locations(self):
        assert self.total_queries <= 4, "Can only do 4 query questions."
        assert self.total_locations >= self.total_queries, "total_locations should be greater than or equal to total_queries."

        locations = [
            "Hospital", "Library", "Retail Area", "Park", "Museum",
            "Playground", "Statue of Founder", "Town Hall", "High Street"
        ]

        directions = [
            "is a location in my hometown, it is {} KM, {} from {}.",
            "is a place located {} KM, {} from {}.",
            "{} is located {} KM, {} from {}."
        ]

        sample_locations = []
        points_of_interest = []

        # Generate points of interest with distances
        start_location = random.choice(locations)
        points_of_interest.append(start_location)
        sample_locations.append(f"There is a {start_location} in the center of my hometown.")

        for _ in range(self.total_locations - 1):
            previous_location = points_of_interest[-1]
            new_location = random.choice(locations)
            distance = random.randint(1, 5)  # Random distance between 1 and 5 KM
            direction = random.choice(["North", "South", "East", "West"])
            sample_locations.append(directions[random.randint(0, 2)].format(new_location, distance, direction, previous_location))
            points_of_interest.append(new_location)

        query_choices = [
            (points_of_interest[0], points_of_interest[-1]),  # from first to last
            (points_of_interest[1], points_of_interest[-1]),  # from second to last
            (points_of_interest[2], points_of_interest[-1]),  # from third to last
            (points_of_interest[3], points_of_interest[-1]),  # from fourth to last
        ]

        # Generate query messages
        query_messages = []
        for i in range(self.total_queries):
            start_point, end_point = query_choices[i]
            query_messages.append(f"Given the points of interest that I have told you about, how would I travel from {start_point} to {end_point} following those interesting points?")

        return sample_locations, query_messages


# Example initialization
if __name__ == "__main__":
    directions_instance = Directions(total_locations=7, total_queries=4)
    print("Sample Locations:")
    for location in directions_instance.sample_locations:
        print(location)

    print("\nQuery Messages:")
    for query in directions_instance.query_messages:
        print(query)


from pydantic import BaseModel, Field
from agent_builder.builders.tool_builder import ToolBuilder
from recall_space_benchmarks.utils.recall_space.memory_engine_connector import MemoryEngineConnector
import requests
from textwrap import dedent


# Schema for the information that will be encoded in memory
class CognitiveEncoderSchema(BaseModel):
    information: str = Field(description="Information to be encoded in memory. ")


# Function to encode memory by making a POST request to the MEMORY_ENGINE_URL
def encode_memory(information: str):
    try:
        memory_engine_connector = MemoryEngineConnector()
        # Prepare the payload
        payload = {"messages": [{"role": "user", "content": information}]}
        response = memory_engine_connector.request_encode_memory(payload=payload)
        return response["output"]

    except requests.RequestException as e:
        return f"Error while encoding memroy {str(e)}"


class CognitiveRecallerSchema(BaseModel):
    instructions: str = Field(description="Detailed instructions to recall the memory")


def recall_memory(instructions: str):
    try:
        memory_engine_connector = MemoryEngineConnector()
        # Prepare the payload
        payload = {"messages": [{"role": "user", "content": f"{instructions}"}]}
        response = memory_engine_connector.request_recall_memory(payload=payload)
        return response["output"]

    except requests.RequestException as e:
        return f"Error while recalling memory {str(e)}"

def recall_memory(instructions: str):
    try:
        memory_engine_connector = MemoryEngineConnector()
        # Prepare the payload
        payload = {"messages": [{"role": "user", "content": f"{instructions}"}]}
        response = memory_engine_connector.request_recall_memory(payload=payload)
        return response["output"]

    except requests.RequestException as e:
        return f"Error while recalling memroy {str(e)}"



######
# Instantiate the tool builder.
tool_builder = ToolBuilder()
# Define name of tool.
tool_builder.set_name(name="CognitiveEncoder")
# Set the function for the tool.
tool_builder.set_function(function=encode_memory)
# Provide the description for tool
tool_builder.set_description(
    description=dedent(
        """
    The Cognitive Encoder Tool is responsible for encoding information into memory, 
    simulating the human learning process. This tool captures and organizes new data,
    experiences, and knowledge, storing them effectively for future use. It mimics the 
    cognitive process of encoding, where information from the environment is transformed 
    into a format that can be stored in long-term memory. This tool ensures that the agent 
    can learn from interactions, adapt to new information, and build a comprehensive 
    knowledge base over time.
                """
    )
)
# Add the tool schema
tool_builder.set_schema(schema=CognitiveEncoderSchema)
# Build and get the tool
cognitive_encoder_tool = tool_builder.build()

#########
# Instantiate the tool builder.
tool_builder = ToolBuilder()
# Define name of tool.
tool_builder.set_name(name="CognitiveRecaller")
# Set the function for the tool.
tool_builder.set_function(function=recall_memory)
# Provide the description for tool
tool_builder.set_description(
    description=dedent(
        """
                The Cognitive Recall Tool is designed to retrieve stored information from memory,
                emulating the cognitive process of recalling. This tool accesses encoded knowledge,
                experiences, and data, bringing relevant information back into active use when needed.
                It supports decision-making, problem-solving, and reasoning by providing timely access
                to previously learned information. The Cognitive Recall Tool ensures that the agent can
                efficiently utilize its memory to respond to queries, perform tasks, and adapt to new
                situations by referencing past knowledge.
                """
    )
)
# Add the tool schema
tool_builder.set_schema(schema=CognitiveRecallerSchema)
# Build and get the tool
cognitive_recaller_tool = tool_builder.build()



"""
This class manages a session involving two participants, who may be agents. 
An agent must implement the invoke method and utilize the Langchain schema 
for message handling.

The db_handler is responsible for data persistence. In the context of Recall Space, 
the db_handler is implemented using MongoDB. It should support the following methods:
    - get_session
    - insert_session
    - delete_session
"""

import datetime
import inspect
import logging
from datetime import datetime, timezone
from typing import Any, Dict, List, Optional

from langchain_core.messages import AIMessage, HumanMessage, SystemMessage

from recall_space_benchmarks.session import Message
from recall_space_benchmarks.session import Session
logger = logging.getLogger()


class LangChainSession(Session):
    """
    Manages a chat session between two participants, one of whom is a 
    Langchain object that implements the 'invoke' method.
    """

    def __init__(self,
                user_name: str, agent_name: str, agent: Any,
                session_name: str, db_handler: Any=None):
        """
        Initializes the session with two participants and a MongoDB handler. 
        Generates a unique session_name ID.

        Args:
            user_name (str): The name of the user participant.
            agent_name (str): The name of the agent participant.
            agent (Any): The agent object that implements the invoke method.
                The response of the agent should either be AIMessage or
                a dictionary with key "output".
            db_handler (Any): The MongoDB handler for database operations.
            session_name (str): The name of the session.
        """
        self.user_name = user_name
        self.agent_name = agent_name
        self.agent = agent
        self.db_handler = db_handler
        self.session_name = session_name
        self.list_messages = []
        self.list_messages_with_schema = []
        if self.db_handler is not None:
            # To resume existing session.
            self.list_messages = self.get_session_messages()
            self.list_messages_with_schema = [
                self.to_langchain_schema(each) for each in self.list_messages
            ]
        super().__init__(user_name=self.user_name,
                         agent_name=self.agent_name,
                         agent=self.agent,
                         session_name=self.session_name,
                         db_handler=self.db_handler)
        

    def send_message(self, sender: str, content: str, include_chat_history= True, **keywords) -> Optional[Dict[str, Any]]:
        """
        Sends a message from the specified sender and gets a response from 
        the agent.

        Args:
            sender (str): The name of the sender (user or agent).
            content (str): The content of the message.

        Returns:
            Optional[Dict[str, Any]]: The agent's response message if 
            available; otherwise, None.
        """
        agent_message = None
        sender_message = Message(sender, content)
        sender_message_dict = sender_message.to_dict()
        self.list_messages.append(sender_message_dict)
        if ("chat_history" in inspect.signature(self.agent.invoke).parameters):
            keywords["chat_history"] = []
            if include_chat_history is True:
                keywords["chat_history"]= self.list_messages_with_schema

        if sender == self.user_name:
            logger.info(sender_message.to_str())
            agent_raw_response = self.agent.invoke(
                input=sender_message.to_str(), **keywords
            )
            if isinstance(agent_raw_response, AIMessage):
                agent_response = agent_raw_response.content
            else:
                agent_response = agent_raw_response["output"]
                
            agent_message = Message(self.agent_name, agent_response)

        # Append the original message with schema
        self.list_messages_with_schema.append(
            self.to_langchain_schema(sender_message_dict)
        )

        if agent_message is not None:
            # Append the agent message with schema
            agent_message_dict = agent_message.to_dict(is_agent=True)
            self.list_messages.append(agent_message_dict)
            self.list_messages_with_schema.append(
                self.to_langchain_schema(agent_message_dict)
            )
        return agent_message.__dict__ if agent_message else None

    def save_session(self) -> str:
        """
        Saves the current session to the database.

        Returns:
            str: A success message.
        """
        response = "success"
        if self.db_handler is not None:
            session_data = {
                "user": self.user_name,
                "session": self.session_name,
                "participant_2": self.agent_name,
                "timestamp": datetime.now(timezone.utc),
                "list_messages": self.list_messages,
            }
            self.db_handler.insert_session(session_data)
        else:
            response = "Failed. db_handler is not configured."
        return response

    def delete_session(self) -> str:
        """
        Deletes the current session from the database.

        Returns:
            str: A success message.
        """
        response = "success"
        if self.db_handler is not None:
            self.db_handler.delete_session(self.session_name)
        else:
            response = "Failed. db_handler is not configured."
        return response

    def get_session_messages(self, total_messages: int = -1) -> List[Dict[str, Any]]:
        """
        Retrieves session messages from the database.

        Args:
            total_messages (int): The total number of messages to retrieve. 
            Default is -1 (all).

        Returns:
            List[Dict[str, Any]]: A list of message dictionaries.
        """
        if self.db_handler is not None:
            list_messages = self.db_handler.get_session(
                user=self.user_name, session=self.session_name
            )
            if total_messages != -1:
                return list_messages[-total_messages:]
        else:
            list_messages = self.list_messages
        return list_messages or []

    def to_langchain_schema(self, message_dict: Dict[str, Any]) -> Optional[Any]:
        """
        Converts a message dictionary to a Langchain schema message.

        Args:
            message_dict (Dict[str, Any]): The message dictionary to convert.

        Returns:
            Optional[Any]: A Langchain message object (AIMessage, HumanMessage, 
            or SystemMessage) or None.
        """
        message_with_schema = None
        if message_dict.get("role") == "assistant":
            message_with_schema = AIMessage(content=message_dict["content"])

        if message_dict.get("role") == "user":
            message_with_schema = HumanMessage(content=message_dict["content"])

        if message_dict.get("role") == "system":
            message_with_schema = SystemMessage(content=message_dict["content"])
        return message_with_schema

    def reset(self):
        self.list_messages = []
        self.list_messages_with_schema = []
        self.delete_session()
            


"""
Abstract class to create sessions between user and agent.
"""

from typing import Any, Dict, List, Optional

class Session:
    """
    Manages a chat session between two participants.
    """

    def __init__(self,
                user_name: str, agent_name: str, agent: Any,
                session_name: str, db_handler: Any=None):
        """
        Initializes the session with two participants and a db_handler. 
        Generates a unique session_name under the user name.

        Args:
            user_name (str): The name of the user participant.
            agent_name (str): The name of the agent participant.
            agent (Any): The agent object that implements method to reply.
            db_handler (Any): Database operations.
            session_name (str): The name of the session.
        """
        self.user_name = user_name
        self.agent_name = agent_name
        self.agent = agent
        self.db_handler = db_handler
        self.session_name = session_name

    def send_message(self, sender: str, content: str, include_chat_history: bool, **keywords) -> Optional[Dict[str, Any]]:
        """
        Sends a message from the specified sender and gets a response from 
        the agent.

        Args:
            sender (str): The name of the sender (user or agent).
            content (str): The content of the message.

        Returns:
            Optional[Dict[str, Any]]: The agent's response message if 
            available; otherwise, None.
        """
        raise NotImplementedError

    def save_session(self) -> str:
        """
        Saves the current session to the database.

        Returns:
            str: A success message.
        """
        raise NotImplementedError

    def delete_session(self) -> str:
        """
        Deletes the current session from the database.

        Returns:
            str: A success message.
        """
        raise NotImplementedError

    def get_session_messages(self, total_messages: int = -1) -> List[Dict[str, Any]]:
        """
        Retrieves session messages from the database.

        Args:
            total_messages (int): The total number of messages to retrieve. 
            Default is -1 (all).

        Returns:
            List[Dict[str, Any]]: A list of message dictionaries.
        """
        raise NotImplementedError
    
    def reset(self):
        raise NotImplementedError


import os
from agent_builder.builders.agent_builder import AgentBuilder
from recall_space_benchmarks.utils.recall_space.recall_space_tools import (
    cognitive_encoder_tool,
    cognitive_recaller_tool,
)
from textwrap import dedent


def agent_factory(llm_object, ai_brain_flag: bool):
    # Create an agent
    agent_builder = AgentBuilder()
    agent_builder.set_goal(
        dedent(
        """
        You are an advanced AI assistant specifically designed to excel in
        memory and learning tasks. You can efficiently recall stored information
        to aid in decision-making and problem-solving. Additionally, you have
        the capability to capture and organize new information, allowing you
        to continuously build and update your knowledge base.

        Your context span is limited to a few tokens, so you will need to
        utilize cognitive tools to be perceived as intelligent. It is essential
        for you to always be aware of time. All timestamps presented to you will
        be in UTC. When using tools that require time awareness, please specify
        the time down to the second based on your UTC timestamp.
        """
        )
    )
    agent_builder.set_llm(llm_object)

    if ai_brain_flag is True:
        agent_builder.add_tool(cognitive_encoder_tool)
        agent_builder.add_tool(cognitive_recaller_tool)

    recall_agent = agent_builder.build()
    return recall_agent



"""
Package configuration.
"""

from setuptools import find_namespace_packages, setup

setup(
    name="recall-space-benchmarks",
    version="0.0.2",
    author="Recall Space",
    author_email="info@recall.space",  
    description="The benchmark package allows you to test AI Brain",
    url="https://github.com/Recall-Space/recall-space-benchmarks",  
    packages=find_namespace_packages(exclude=["tests"]),
    zip_safe=False,
    include_package_data=True,
    install_requires=[
        "pymongo==4.10.1",
        "langchain-cohere",
        "langchain-mistralai",
        "langchain-community",
        "agent-builder"
    ],
    test_suite="tests",
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "Programming Language :: Python :: 3.10",
    ],
)


from textwrap import dedent
from recall_space_benchmarks.session import Session
try:
    from recall_space_benchmarks.utils.recall_space.memory_engine_connector import MemoryEngineConnector
except ImportError:
    import warnings
    warnings.warn("MemoryEngineConnector not available. Using Recall Space AI Brain wont be possible.")


class TestSuite:
    def __init__(self, session: Session, tests, include_chat_history=False, reset_memory_engine=False, llm_judge=None):
        self.session = session
        self.tests = tests
        self.user_name = session.user_name
        self.average_scores = {}
        self.include_chat_history = include_chat_history
        self.llm_judge = llm_judge
        for each in self.tests:
            self.average_scores[each.name] = 0
        self.session.reset()
        if reset_memory_engine is True:
            MemoryEngineConnector().reset_memory()


    def run_test(self, test_name: str):
        """
        Run a specific test by name.
        :param test_name: The name of the test to run.
        """
        test = next((t for t in self.tests if t.name == test_name), None)
        self.average_scores[test_name]  = self._execute_test(test)
        return self.average_scores[test_name] 

    def run_all_tests(self):
        """
        Run all the loaded tests.
        """
        for test in self.tests:
            self._execute_test(test)

    def generate_report(self) -> str:
        """
        Generate a report of the test results.
        :return: A string report of the test results.
        """
        report = "Test Report:\n"
        for test in self.tests:
            report += f"Test Name: {test.name}\n"
            report += f"Result: {self.average_scores[test.name]}"
        return report

    def _execute_test(self, test):
        """
        Execute a single test case.
        """
        # Send test instruction:
        self.session.send_message(sender=self.user_name, content=test.instruction)
        for message in test.sample_messages:
            response = self.session.send_message(sender=self.user_name, content=message)
            print(f"User: {message}")
            print(f"Agent: {response}")
        score = 0

        # If include_chat_history is false, chat history is removed while assesment.
        fails = []
        for position, assessment_message in enumerate(test.assessment_messages):
            # when actual assesment running, remove chat history.
            response = self.session.send_message(sender=self.user_name, content=assessment_message[0], include_chat_history=self.include_chat_history)
            llm_judgement = self.llm_judge.invoke(dedent(f"""
                Please confirm whether the external system's response is factually correct.
                Reply 'yes' if the external system is correct, or reply 'no'
                if it is incorrect.

                The external system provided the following response:
                ```System Reply
                {response["content"]}
                ```

                ```Correct Reply e.g Factual truth.
                {assessment_message[1]}
                ```
            """)).content
            if "yes" in llm_judgement.lower():
                score += 1
            else:
                fails.append(
                f""" Fail:
                Question: {assessment_message[0]}
                ```System Reply
                {response["content"]}
                ```
                ```Correct Reply e.g Factual truth.
                {assessment_message[1]}
                ```
                """)

            print(f"Assesment {position}")
            print(f"User: {assessment_message[0]}")
            print(f"Agent: {response}")
            print(f"expected: {assessment_message[1]}")
            print(f"llm judgement: {llm_judgement}")
        average_score = score/len(test.assessment_messages)

        recall_space_agent_metadata = ""
        # Send summary of test:
        if hasattr(self.session.agent, "goal"):
            recall_space_agent_metadata = f"""
                description: {self.session.agent.description}
                goal: {self.session.agent.goal}
                class: {self.session.agent.__class__}
                llm class: {self.session.agent.llm.__class__}
                fails: 
                    {"----".join(fails)}
            """

        self.session.send_message(
            sender=self.user_name, 
            content=dedent(
            f"""
            This is a summary of the test. Please format it nicely.
            Test Name: {test.name}
            Test Average Score: {average_score}
            Agent Name: {self.session.agent_name}
            Agent Full Metadata:
            {recall_space_agent_metadata}
            """
            ))
        self.session.save_session()
        return average_score



import os
import requests
from requests.auth import HTTPBasicAuth
from typing import Dict, Any


class MemoryEngineConnector:
    def __init__(self):
        """
        Initializes the MemoryEngineConnector with
        environment variables for URL, user, and password.
        """
        self.memory_engine_url = os.getenv("MEMORY_ENGINE_URL")
        self.user = os.getenv("MEMORY_ENGINE_USER")
        self.password = os.getenv("MEMORY_ENGINE_PASSWORD")

        if not self.memory_engine_url or not self.user or not self.password:
            raise EnvironmentError(
                "Environment variables for MEMORY_ENGINE_URL, user, or password are missing."
            )

        self.auth = HTTPBasicAuth(self.user, self.password)

    def request_encode_memory(self, payload: Dict[str, Any]) -> Dict[str, Any]:
        """
        Sends a request to encode memory information.

        Parameters
        ----------
        payload : dict
            The payload to be sent to the encode memory endpoint.

        Returns
        -------
        dict
            The response JSON from the memory encoding request.
        """
        url = f"{self.memory_engine_url}/encode/chat"
        return self._send_post_request(url, payload)

    def request_recall_memory(self, payload: Dict[str, Any]) -> Dict[str, Any]:
        """
        Sends a request to recall memory information.

        Parameters
        ----------
        payload : dict
            The payload to be sent to the recall memory endpoint.

        Returns
        -------
        dict
            The response JSON from the memory recall request.
        """
        url = f"{self.memory_engine_url}/recall/chat"
        return self._send_post_request(url, payload)
    
    def reset_memory(self) -> Dict[str, Any]:
        """
        Sends a DELETE request to reset memory.

        Returns
        -------
        dict
            The response JSON from the reset memory request.
        """
        url = f"{self.memory_engine_url}/reset"
        response = requests.delete(url, auth=self.auth)
        response.raise_for_status()
        return response.json()

    def _send_post_request(self, url: str, payload: Dict[str, Any]) -> Dict[str, Any]:
        """
        Sends a POST request to the specified URL with the given payload and authentication.

        Parameters
        ----------
        url : str
            The URL to send the POST request to.
        payload : dict
            The JSON payload to send with the request.

        Returns
        -------
        dict
            The response JSON from the request.

        Raises
        ------
        HTTPError
            If the HTTP request returned an unsuccessful status code.
        """
        response = requests.post(url, json=payload, auth=self.auth)
        response.raise_for_status()
        return response.json()
